import subprocess
import re
from . import util
import moyanlib


def get_version(command):
    try:
        # 使用 subprocess 调用系统命令获取版本信息
        output = subprocess.check_output(
            command, shell=True, stderr=subprocess.STDOUT, universal_newlines=True
        )
        # 使用正则表达式提取版本号
        version_match = re.search(r"(\d+\.\d+(\.\d+){0,2})", output)
        if version_match:
            return version_match.group(1)
        else:
            return "[red]False[/red]"
    except subprocess.CalledProcessError:
        return "[red]False[/red]"


def getGCCver():
    # 检查是否能执行 gcc 命令
    return get_version("gcc --version")


def getClangVer():
    # 检查是否能执行 clang 命令
    return get_version("clang --version")


def getMSVCver():
    # 检查是否能执行 cl 命令（MSVC编译器）
    return get_version("cl")


def getRustVer():
    process = subprocess.Popen(
        ["rustc --version"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True
    )
    stdout, stderr = process.communicate()
    if process.returncode == 0:
        rustVer = stdout.decode().split(" ")[1]
        return rustVer
    else:
        return "[red]False[/red]"


def getPIPver():
    return get_version("pip -V")


def main():
    info = moyanlib.getInfo()
    printText = f"""[white]PMPT {util.__version__}[/white]
{info['OS']['Name']} {info['OS']['Version']}
Python Version: {info['Python']['Version']}
PIP Version: [green]{getPIPver()}[/green]
GCC Version: [green]{getGCCver()}[/green]
Clang Version: [green]{getClangVer()}[/green]
MSVC Version: [green]{getMSVCver()}[/green]
Rust Version: [green]{getRustVer()}[/green]"""
    util.console.print(printText)



# pylint:disable=W0622
import os
from subprocess import Popen
import sys
import dill
from rich.console import Console
from loguru import logger
import subprocess
from moyanlib import jsons
import time
import datetime
from platformdirs import PlatformDirs

dirs = PlatformDirs("PMPT", "MoYan")
IndexList = []
console = Console()


def getVer(baseVar):
    if os.environ.get('GITHUB_ACTIONS', None):
        baseVar = (
            baseVar + '.' + os.environ.get('GITHUB_RUN_NUMBER')
        )  
    logger.info("PMPT " + baseVar)
    return baseVar


logger.remove()
logger.add(
    os.path.join(dirs.user_data_dir, "log.log"),
    level="TRACE",
)

__version__ = getVer("1.0.5")


def init():

    os.makedirs(dirs.user_data_dir, exist_ok=True)
    os.makedirs(os.path.join(dirs.user_data_dir, "Index"), exist_ok=True)
    os.makedirs(dirs.user_config_dir, exist_ok=True)
    os.makedirs(dirs.user_cache_dir, exist_ok=True)
    if not os.path.exists(os.path.join(dirs.user_config_dir, "Source.json")):
        open(os.path.join(dirs.user_config_dir, "Source.json"), "w").write("[]")
        from . import source

        source.add('https://pypi.org/simple',1)

    if not os.path.exists(os.path.join(dirs.user_config_dir, "api.url")):
        open(os.path.join(dirs.user_config_dir, "api.url"), "w").write(
            "https://pypi.org/pypi/{}/json"
        )


def bubbleSort(arr):
    for i in range(1, len(arr)):
        for j in range(0, len(arr) - i):
            if arr[j]["priority"] < arr[j + 1]["priority"]:
                arr[j], arr[j + 1] = arr[j + 1], arr[j]
    return arr


def loadIndex():
    """
    加载索引
    """
    today = datetime.datetime.today()
    if today.month == 4 and today.day == 1:
        console.print(
            "😱😱😱 [red]Oh no, your indexes have all been eaten by a python snake.[/red]"
        )
        time.sleep(2.3)
        console.print(
            "😄 [green]But after our life-and-death struggle, I managed to retrieve all of your indexes.[/green]"
        )
    sourceList = jsons.load(open(os.path.join(dirs.user_config_dir, "Source.json")))
    sourceList = bubbleSort(sourceList)
    for source in sourceList:
        if not os.path.exists(
            os.path.join(dirs.user_data_dir, "Index", source["id"] + ".pidx")
        ):
            logger.warning(source["url"] + "没有索引")
            console.print(
                f'⚠️ [yellow]{source["url"]} did not create an index.[/yellow]'
            )
            continue
        logger.debug(source)
        IndexFile = dill.load(
            open(
                os.path.join(dirs.user_data_dir, "Index", source["id"] + ".pidx"), "rb"
            )
        )  # 加载索引
        yield IndexFile


def runpip(command, other=None, dbg=False, out=True) -> Popen:
    """
    运行pip
    """
    logger.trace("调用runpip")
    if not other:
        other = []
    baseCommand = [sys.executable, "-m", "pip"]
    baseCommand.append(command)

    Command = baseCommand + other
    if dbg:
        console.print("Command to be run:", " ".join(Command))
    logger.debug(
        " ",
    )
    runClass = Popen(Command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    if out:
        for line in iter(runClass.stdout.readline, b""):
            # 在这里你可以对每一行输出进行处理
            line = line.decode("utf-8").strip()  # 将字节转换为字符串并去除换行符
            console.print(line)
        if runClass.returncode != 0:
            console.print(runClass.stderr.read().decode())
        runClass.communicate()
    else:
        runClass.wait()
    return runClass



from tinydb import TinyDB, Storage, Query

# from . import util
import os
import warnings
from moyanlib import jsons
import io
from tqdm import tqdm
import zstandard
from typing import Dict, Any, Optional


class FastJSONStorage(Storage):
    def __init__(
        self,
        path: str,
        create_dirs=False,
        encoding=None,
        access_mode="rb+",
        write_threshold=1,
        **kwargs
    ):
        super().__init__()
        self.cctx = zstandard.ZstdCompressor()
        self.dctx = zstandard.ZstdDecompressor()
        self._mode = access_mode
        self.kwargs = kwargs
        self.write_threshold = write_threshold  # 指定写入阈值
        self.write_counter = 0
        if any(
            [character in self._mode for character in ("+", "w", "a")]
        ):  # any of the writing modes
            self.touch(path, create_dirs=create_dirs)
        self._handle = open(path, mode=self._mode, encoding=encoding)

    def close(self) -> None:
        self._handle.close()

    def touch(self, path: str, create_dirs: bool):
        if create_dirs:
            base_dir = os.path.dirname(path)

            if not os.path.exists(base_dir):
                os.makedirs(base_dir)

        with open(path, "a"):
            pass

    def read(self) -> Optional[Dict[str, Dict[str, Any]]]:
        self._handle.seek(0, os.SEEK_END)
        size = self._handle.tell()

        if not size:
            return None
        else:
            self._handle.seek(0)
            data = self.dctx.decompress(self._handle.read())
            return jsons.loads(data.decode())

    def write(self, data: Dict[str, Dict[str, Any]]):
        self.write_counter += 1
        if self.write_counter == self.write_threshold:
            self._handle.seek(0)

            serialized = jsons.dumps(data, **self.kwargs)
            serialized = self.cctx.compress(serialized.encode())

            try:
                self._handle.write(serialized)
            except io.UnsupportedOperation:
                raise IOError(
                    'Cannot write to the database. Access mode is "{0}"'.format(
                        self._mode
                    )
                )

            self._handle.flush()
            os.fsync(self._handle.fileno())

            self._handle.truncate()
            self.write_counter = 0


class Base:
    def __init__(self, name):
        self.rootdb = TinyDB(
            os.path.join(".", "DB.pcdb"), storage=FastJSONStorage, write_threshold=2
        )
        self.db = self.rootdb.table(name)

    def insert(self, data):
        self.db.insert(data)

    def remove(self, query):
        self.db.remove(query)


class PackageData(Base):
    def __init__(self):
        super(PackageData, self).__init__("PackageData")
        self.query = Query()

    def add(self, name, version, info=None):
        self.db.insert({"Name": name, "Version": version, "info": info})

    def get(self, name):
        return self.db.search(self.query.Name == name)[0]


db = PackageData()
for i in tqdm(range(1145)):
    db.add("json" + str(i), "v1" + str(i))

print(db.db.all())



import uuid
from moyanlib import jsons
import os
from rich.table import Table
from . import util


def line_search(li, val, key):
    # 线性搜索算法
    n = 0
    for i in li:
        print(i)
        if i[key] == val:
            return i, n
        n += 1
    return None


def add(url, priority):
    """
    添加源
    """
    sourceList = jsons.load(
        open(os.path.join(util.dirs.user_config_dir, "Source.json"))
    )  # 加载source源

    if not line_search(sourceList, url, "url"):  # 判断源是否存在
        util.console.print("❌ [red]The source already exists[/red]")
        exit(1)

    sourceList.append({"url": url, "id": str(uuid.uuid4())[:8], "priority": priority})
    jsons.dump(
        sourceList, open(os.path.join(util.dirs.user_config_dir, "Source.json"), "w")
    )


def lists():
    """
    列出所有源
    """
    sourceList = jsons.load(
        open(os.path.join(util.dirs.user_config_dir, "Source.json"))
    )
    # 构建table
    table = Table(show_header=True)
    table.add_column("ID", width=6)
    table.add_column("Priority", width=8)
    table.add_column("URL", width=25)

    # 遍历source列表
    for i in sourceList:
        table.add_row(i["id"], str(i["priority"]), i["url"])
    util.console.print(table)


def remove(ids, yes):
    """
    删除
    """
    if not ids:
        lists()
        ids = util.console.input(
            "Please enter the source sequence number to be deleted:"
        )

    sourceList: list = jsons.load(
        open(os.path.join(util.dirs.user_config_dir, "Source.json"))
    )
    source, n = line_search(sourceList, ids, "id")

    if not source:
        util.console.print("❌ [red]The source for this ID does not exist[/red]")

    while True:  # 是否允许安装
        if yes:
            break

        ye = util.console.input("Are you sure you want to delete? [Y/n]: ")

        if ye.lower() == "y":
            break
        elif ye.lower() == "n":
            util.console.print("🛑 [red]The user exited the program[/red]")
            exit(1)
        else:
            continue

    sourceList.remove(source)
    jsons.dump(
        sourceList, open(os.path.join(util.dirs.user_config_dir, "Source.json"), "w")
    )


def modify(ids, key, val):
    """
    修改
    """
    sourceList: list = jsons.load(
        open(os.path.join(util.dirs.user_config_dir, "Source.json"))
    )
    source, n = line_search(sourceList, ids, "id")

    if not source:
        util.console.print("❌ [red]The source for this ID does not exist[/red]")

    if key not in ["url", "priority"]:
        util.console.print(
            "❌ [red]The item is not allowed to be modified or does not exist.[/red]"
        )

    sourceList[n][key] = val

    jsons.dump(
        sourceList, open(os.path.join(util.dirs.user_config_dir, "Source.json"), "w")
    )



from . import util
import requests
from rich.table import Table
from urllib import parse


def search(name):
    for Index in util.loadIndex():
        dt = Index.packageList.get(name, None)
        if dt:
            try:
                rurl = parse.urljoin(Index.IndexURL, name)
                r = requests.get(rurl)
            except:
                continue
            else:
                if r.status_code != 200:
                    continue
            return Index.IndexURL


def main(
    packlist, upgrade, reads, force_reinstall, ignore_requires_python, yes, command
):
    console = util.console
    if reads:  # 从文件读取列表
        f = open(packlist[0])
        packlist = f.read().split("\n")

    packsInfo = {}
    with console.status("🚀🔍 Searching for package information...") as status:
        # 创建表格
        table = Table(show_header=True, header_style="bold")
        table.add_column("Package Name", width=20, style="bold")
        table.add_column("Package Source", width=20, style="green")

        for rawpack in packlist:  # 解析指定了版本的包名
            if "==" in rawpack:
                pack = rawpack.split("==")[0]
            elif ">=" in rawpack:
                pack = rawpack.split(">=")[0]
            elif "<=" in rawpack:
                pack = rawpack.split("<=")[0]
            elif "<" in rawpack:
                pack = rawpack.split("<")[0]
            elif ">" in rawpack:
                pack = rawpack.split(">")[0]
            else:
                pack = rawpack

            result = search(pack.lower())  # 转小写并获取源地址
            packsInfo[pack] = [result, rawpack]

    canInstallPack = []
    for k, v in packsInfo.items():
        if not v[0]:
            table.add_row(k, "[red]Not found[red]")
        else:
            table.add_row(k, v[0])
            canInstallPack.append(k)
    console.print(table)

    if len(canInstallPack) < 1:
        console.print("❌ [red]There are no packages available for installation.[red]")
        exit(1)

    console.print("📦 Packages to be installed:")
    console.print(" ".join(canInstallPack))

    while True:  # 是否允许安装
        if yes:
            break

        ye = console.input("Proceed with installation? [Y/n]: ")

        if ye.lower() == "y":
            break
        elif ye.lower() == "n":
            console.print("🛑 [red]User canceled the installation.[/red]")
            exit(1)
        else:
            continue

    console.print("🛠️ Initiating pip installation...")
    for pack in canInstallPack:

        # 构建命
        args = ["-i", packsInfo[pack][0]]  # 指定源

        if upgrade:  # 升级
            args.append("-U")
        if force_reinstall:  # 强制重新安装
            args.append("--force-reinstall")
        if ignore_requires_python:  # 忽略Python版本
            args.append("--ignore-requires-python")

        args.append(packsInfo[pack][1])
        with console.status("🚀 [green]Installing...[/green]") as status:
            ret = util.runpip("install", args, command)  # 运行pip

        if ret.returncode != 0:  # 是否执行完毕
            console.print("❌ [red]Installation failed.[/red]")
            exit(1)
        console.print(f"✅ [green]Installation successful for {pack}[/green]")



import requests
from moyanlib import jsons
from tqdm import tqdm
import dill
import os
from .util import dirs, console

packageNum = 0


def getSourceID(url):
    """
    获取源id
    """
    return url["id"]  # 取前八位


class Index:
    def __init__(self, indexurl):
        self.packageList = {}
        self.IndexURL = indexurl["url"]
        self.number = 0
        self.priority = indexurl["priority"]

    def addPackage(self, name):
        self.packageList[name] = "1"
        self.number += 1


def getIndex(url):
    global packageNum
    try:
        req = requests.get(url["url"])  # 请求HTML
    except Exception:
        console.print("[red]Unable to connect to source[/red]")
        return False
    HTMLIndex = req.text

    ClassIndex = Index(url)

    console.print("🔍 Parsing HTML index...")
    for line in tqdm(HTMLIndex.split("\n")):
        # 提取并筛选标签
        line_list = line.split(">")
        if len(line_list) > 1 and "<a " in line:
            package_name = line_list[1].split("<")[0]

            ClassIndex.addPackage(package_name)  # 添加包

    console.print("Total number of packages:", str(ClassIndex.number))
    packageNum += ClassIndex.number
    console.print('📚 Saving index..."')
    dill.dump(
        ClassIndex, open(f"{dirs.user_data_dir}/Index/{getSourceID(url)}.pidx", "wb")
    )
    return True


def getAllIndex():
    """
    SourceList = [
        'https://pypi.tuna.tsinghua.edu.cn/simple',
        'https://mirrors.bfsu.edu.cn/pypi/web/simple/'
    ]
    """
    SourceList = jsons.load(
        open(os.path.join(dirs.user_config_dir, "Source.json"))
    )  # 加载源列表
    if len(SourceList) < 1:
        console.print("❌ [red]You have not configured any sources.[/red]")
        exit(1)

    for source in SourceList:  # 遍历源列表
        console.print("📚 Downloading index from", source["url"] + "...")
        sta = getIndex(source)
        if sta:
            console.print("✅ [green]Index downloaded successfully![/green]")
        else:
            console.print("❌ [red]Index download failed.[/red]")
    # print(packageNum)



from flask import Flask, render_template, jsonify, redirect
from . import util
import os
from flask_caching import Cache

current_dir = os.path.dirname(__file__)

app = Flask(
    __file__,
    template_folder=os.path.join(current_dir, "data"),
    static_folder=os.path.join(current_dir, "data", "statics"),
)

config = {  # some Flask specific configs
    "CACHE_TYPE": "FileSystemCache",  # Flask-Caching related configs
    "CACHE_DEFAULT_TIMEOUT": 600,
    "CACHE_DIR": util.dirs.user_cache_dir,
}
app.config.from_mapping(config)
cache = Cache(app)


def run():
    app.run(port=14536)


@app.route("/")
@cache.cached(timeout=36000)
def index():
    return render_template("index.html")


@app.route("/package/<name>")
def package(name):
    return redirect("https://pypi.org/project/" + name)


@app.route("/api/list")
@cache.cached(timeout=600)
def api_list():
    listv = util.runpip("freeze", out=False)
    outlist = []
    for line in iter(listv.stdout.readline, b""):

        line = line.decode("utf-8").strip()  # 将字节转换为字符串并去除换行符
        if "==" not in line or line[0] == "#":
            continue
        lineList = line.split("==")
        lineDict = {"name": lineList[0], "ver": lineList[1]}
        outlist.append(lineDict)
    return jsonify(outlist)



import requests
from . import util
from moyanlib import jsons
from .install import search
import os

console = util.console


def main(name, allinfo, api_url):
    if not search(name):
        console.print("❌ [red]The package does not exist[/red]")
        exit()

    if not api_url:
        api_url = open(os.path.join(util.dirs.user_config_dir, "api.url")).read()

    req = requests.get(api_url.format(name))
    if req.status_code == 404:
        console.print(404)
        console.print("❌ [red]The package does not exist[/red]")
        exit()
    elif req.status_code != 200:
        console.print("❌ [red]Server Error![/red]")
        console.print("[red]Error Code: " + str(req.status_code) + "[/red]")
        exit()

    packageInfo = jsons.loads(req.text)
    if not allinfo:
        print("Package Name:", packageInfo["info"]["name"])
        print("Version:", packageInfo["info"]["version"])
        print("Author:", packageInfo["info"]["author"])
        print("Summary:", packageInfo["info"]["summary"])
        print("Keywords:", packageInfo["info"]["keywords"])
        print("License:", packageInfo["info"]["license"])
        if packageInfo["info"]["requires_dist"]:
            print("Dependent Library:", ", ".join(packageInfo["info"]["requires_dist"]))
    elif allinfo:
        for k, v in packageInfo["info"].items():
            print(f"{k}: {v}")



# pylint:disable=W0611
import datetime
import webbrowser
import time
import os
import sys
import click
from moyanlib import jsons
from rich.table import Table
from . import update as updates
from . import util
from . import source as sou
from . import environment as environ
from . import install as installs
from . import search as searchs
from . import web as webs


@click.group()
def cli():
    today = datetime.datetime.today()
    if today.month == 9 and today.day == 28:
        webbrowser.open("https://ys.mihoyo.com/")
        util.console.print("[green]Genshin！activate！[/green]")
        time.sleep(0.7)
        util.console.print("[green]You all must download Genshin Impact.[/green]")
        time.sleep(0.7)
        util.console.print("Then don't use PMPT.", style="green")

    try:
        import pip
    except ImportError:
        util.logger.critical("没有pip")
        util.console.print("❌ [red]pip module not found![/red]")
        exit(1)


@cli.command(short_help="Update Package Index")
def update():
    # 调用更新函数
    updates.getAllIndex()


@cli.command(short_help="Install Python package")
@click.argument("packlist", nargs=-1, required=True)
@click.option("--upgrade", "-U", is_flag=True, default=False)
@click.option("--reads", "-r", is_flag=True, default=False)
@click.option("--force-reinstall", "-fr", is_flag=True, default=False)
@click.option("--ignore-requires-python", "-irp", is_flag=True, default=False)
@click.option("--yes", "-y", is_flag=True, default=False)
@click.option("--command", "-c", is_flag=True, default=False)
def install(*args, **kwargs):
    installs.main(*args, **kwargs)


@cli.command(name="list", short_help="List all Python packages")
def listp():
    table = Table(show_header=True)
    table.add_column("Name")
    table.add_column("Version")
    listv = util.runpip("freeze", out=False)
    for line in iter(listv.stdout.readline, b""):

        # 在这里你可以对每一行输出进行处理
        line = line.decode("utf-8").strip()  # 将字节转换为字符串并去除换行符
        if "==" not in line or line[0] == "#":
            continue
        lineList = line.split("==")
        table.add_row(lineList[0], lineList[1])
    util.console.print(table)


@cli.command()
@click.argument("name")
@click.option("--yes", "-y", is_flag=True, default=False)
def remove(name, yes):
    args = []
    if yes:
        args.append("-y")
    args.append(name)
    util.runpip("uninstall", args)


@cli.command()
@click.argument("name")
@click.option("--allinfo", "-a", is_flag=True, default=False)
@click.option("--api-url", "-u", default=None)
def search(*args, **kwargs):
    searchs.main(*args, **kwargs)


@cli.group()
def source():
    pass


@source.command()
@click.argument("url")
@click.option("--priority", "-p", default=1, type=int)
def add(*args, **kwargs):
    sou.add(*args, **kwargs)


@source.command(name="list")
def lists():
    sou.lists()


@source.command(name="remove")
@click.argument("ids", default=None, required=False)
@click.option("-y", "--yes", is_flag=True, default=False)
def removes(*args, **kwargs):
    sou.remove(*args, **kwargs)


@source.command(name="modify")
@click.argument("ids")
@click.argument("key")
@click.argument("val")
def modifys(*args, **kwargs):
    sou.modify(*args, **kwargs)


@cli.command()
def version():
    environ.main()


@cli.command()
def web():
    webs.run()


if __name__ == "__main__":
    cli()



from setuptools import setup, find_packages
from pmpt import util

setup(
    name="pmpt",  # 包的名称
    version=util.__version__,  # 版本号
    packages=find_packages(exclude=["tests", "tests.*"]),  # 包含的包
    author="MoYan",  # 作者
    author_email="moyan@moyanjdc.top",  # 作者邮箱
    description="A Python Package Advanced Manager",  # 包的简要描述
    long_description=open("readme.md").read(),  # 包的详细描述
    long_description_content_type="text/markdown",  # 描述的内容类型
    classifiers=[  # 包的分类信息
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    setup_requires=["platformdirs"],
    install_requires=open("requirements/base.txt").read().split("\n"),
    entry_points={
        "console_scripts": ["pmpt=pmpt:cli"],
    },
    extras_require={
        "web": open("requirements/web.txt").read().split("\n"),
    },
    include_package_data=True,
)
